@@ -108,6 +108,15 @@
    #define BOOST_CONTAINER_FORCEINLINE BOOST_FORCEINLINE
 #endif
 
+//#define BOOST_CONTAINER_DISABLE_NOINLINE
+
+#if defined(BOOST_CONTAINER_DISABLE_NOINLINE)
+   #define BOOST_CONTAINER_NOINLINE
+#else
+   #define BOOST_CONTAINER_NOINLINE BOOST_NOINLINE
+#endif
+
+
 #if !defined(__has_feature)
 #define BOOST_CONTAINER_HAS_FEATURE(feature) 0
 #else
@@ -121,9 +130,34 @@
 
 
 #if (__cplusplus >= 201703L)
-//CTAD supported
+   //CTAD supported
+   #ifdef __INTEL_COMPILER
+      //Intel compilers do not offer this feature yet
+      #define BOOST_CONTAINER_NO_CXX17_CTAD
+   #endif
 #else
-#define BOOST_CONTAINER_NO_CXX17_CTAD
+   #define BOOST_CONTAINER_NO_CXX17_CTAD
 #endif
 
+#if defined(BOOST_CONTAINER_DISABLE_ATTRIBUTE_NODISCARD)
+   #define BOOST_CONTAINER_ATTRIBUTE_NODISCARD
+#else
+   #if   defined(BOOST_GCC) && ((BOOST_GCC < 100000) || (__cplusplus < 201703L))
+      //Avoid using it in C++ < 17 and GCC < 10 because it warns in SFINAE contexts
+      //(see https://gcc.gnu.org/bugzilla/show_bug.cgi?id=89070)
+      #define BOOST_CONTAINER_ATTRIBUTE_NODISCARD
+   #else
+      #define BOOST_CONTAINER_ATTRIBUTE_NODISCARD BOOST_ATTRIBUTE_NODISCARD
+   #endif
+#endif
+
+
+//Configuration options:
+
+//Define this to use std exception types instead of boost::container's own exception types
+//#define BOOST_CONTAINER_USE_STD_EXCEPTIONS
+
+
+
+
 #endif   //#ifndef BOOST_CONTAINER_DETAIL_WORKAROUND_HPP
